/* 
 *  css_descramble.c
 *
 *  Released under the version 2 of the GPL.
 *
 *  Copyright 1999 Derek Fawcus / M Roberts
 *
 *  This file contains functions to descramble CSS encrypted DVD content
 *	
 */


#include <stdio.h>
#include <string.h>

#include "css.h"
#include "css-descramble.h"

#define CSS_DEBUG 0x00

/*
 *
 *  some tables used for descrambling sectors and/or decrypting title keys
 *
 */

static byte csstab1[256]=
{
	0x33,0x73,0x3b,0x26,0x63,0x23,0x6b,0x76,0x3e,0x7e,0x36,0x2b,0x6e,0x2e,0x66,0x7b,
	0xd3,0x93,0xdb,0x06,0x43,0x03,0x4b,0x96,0xde,0x9e,0xd6,0x0b,0x4e,0x0e,0x46,0x9b,
	0x57,0x17,0x5f,0x82,0xc7,0x87,0xcf,0x12,0x5a,0x1a,0x52,0x8f,0xca,0x8a,0xc2,0x1f,
	0xd9,0x99,0xd1,0x00,0x49,0x09,0x41,0x90,0xd8,0x98,0xd0,0x01,0x48,0x08,0x40,0x91,
	0x3d,0x7d,0x35,0x24,0x6d,0x2d,0x65,0x74,0x3c,0x7c,0x34,0x25,0x6c,0x2c,0x64,0x75,
	0xdd,0x9d,0xd5,0x04,0x4d,0x0d,0x45,0x94,0xdc,0x9c,0xd4,0x05,0x4c,0x0c,0x44,0x95,
	0x59,0x19,0x51,0x80,0xc9,0x89,0xc1,0x10,0x58,0x18,0x50,0x81,0xc8,0x88,0xc0,0x11,
	0xd7,0x97,0xdf,0x02,0x47,0x07,0x4f,0x92,0xda,0x9a,0xd2,0x0f,0x4a,0x0a,0x42,0x9f,
	0x53,0x13,0x5b,0x86,0xc3,0x83,0xcb,0x16,0x5e,0x1e,0x56,0x8b,0xce,0x8e,0xc6,0x1b,
	0xb3,0xf3,0xbb,0xa6,0xe3,0xa3,0xeb,0xf6,0xbe,0xfe,0xb6,0xab,0xee,0xae,0xe6,0xfb,
	0x37,0x77,0x3f,0x22,0x67,0x27,0x6f,0x72,0x3a,0x7a,0x32,0x2f,0x6a,0x2a,0x62,0x7f,
	0xb9,0xf9,0xb1,0xa0,0xe9,0xa9,0xe1,0xf0,0xb8,0xf8,0xb0,0xa1,0xe8,0xa8,0xe0,0xf1,
	0x5d,0x1d,0x55,0x84,0xcd,0x8d,0xc5,0x14,0x5c,0x1c,0x54,0x85,0xcc,0x8c,0xc4,0x15,
	0xbd,0xfd,0xb5,0xa4,0xed,0xad,0xe5,0xf4,0xbc,0xfc,0xb4,0xa5,0xec,0xac,0xe4,0xf5,
	0x39,0x79,0x31,0x20,0x69,0x29,0x61,0x70,0x38,0x78,0x30,0x21,0x68,0x28,0x60,0x71,
	0xb7,0xf7,0xbf,0xa2,0xe7,0xa7,0xef,0xf2,0xba,0xfa,0xb2,0xaf,0xea,0xaa,0xe2,0xff
};


/*
 *
 * this function is only used internally when decrypting title key
 *
 */
static void css_titlekey(byte *key, byte *im, byte invert)
{
	int i;
	byte k[5];
	int val;

	unsigned int lfsr0, lfsr1;
	byte         o_lfsr0, o_lfsr1;

	lfsr0 = ((im[4] << 17) | (im[3] << 9) | (im[2] << 1)) + 8 - (im[2] & 7);
       	lfsr0 = (reverse[lfsr0&0xff]<<17) | (reverse[(lfsr0>>8)&0xff] << 9)
		  | (reverse[(lfsr0>>16)&0xff]<<1) |(lfsr0>>24);

	lfsr1 = (reverse[ im[0] ] << 9) | 0x100 | reverse[ im[1]];

#if (CSS_DEBUG & 0x01)
		fprintf( stderr,"SEED lfsr0:0x%08x lfsr1: 0x%08x\n", lfsr0, lfsr1);
#endif

	val = 0;
	for (i = 0; i < 5; ++i) {

   		o_lfsr0 = (lfsr0 >> 12) ^ (lfsr0 >> 4) ^ (lfsr0 >> 3) ^ lfsr0;

		o_lfsr1 = ((lfsr1 >> 14) & 7) ^ lfsr1;
		o_lfsr1 ^= (o_lfsr1 << 3) ^ (o_lfsr1 << 6);

		lfsr1 = (lfsr1 >> 8) ^ (o_lfsr1 << 9);
		lfsr0 = (lfsr0 >> 8) ^ (o_lfsr0 << 17);

#if (CSS_DEBUG & 0x01)
		fprintf( stderr,"lfsr0:0x%08x lfsr1: 0x%08x o_lfsr0:0x%02x o_lfsr1:0x%02x\n",
				lfsr0, lfsr1, o_lfsr0, o_lfsr1);
#endif

		val += (o_lfsr0 ^ invert) + o_lfsr1;

		k[i] = val & 0xff;
		val >>= 8;
	}

	key[4]=k[4]^csstab1[key[4]]^key[3];
	key[3]=k[3]^csstab1[key[3]]^key[2];
	key[2]=k[2]^csstab1[key[2]]^key[1];
	key[1]=k[1]^csstab1[key[1]]^key[0];
	key[0]=k[0]^csstab1[key[0]]^key[4];

	key[4]=k[4]^csstab1[key[4]]^key[3];
	key[3]=k[3]^csstab1[key[3]]^key[2];
	key[2]=k[2]^csstab1[key[2]]^key[1];
	key[1]=k[1]^csstab1[key[1]]^key[0];
	key[0]=k[0]^csstab1[key[0]];
}

/*
 *
 * this function decrypts a title key with the specified disk key
 *
 * tkey: the unobfuscated title key (XORed with BusKey)
 * dkey: the unobfuscated disk key (XORed with BusKey)
 *       2048 bytes in length (though only 5 bytes are needed, see below)
 * pkey: array of pointers to player keys and disk key offsets
 *
 *
 * use the result returned in tkey with css_descramble
 *
 */

int css_decrypttitlekey(byte *tkey, byte *dkey, struct playkey **pkey)
{
	byte test[5], pretkey[5];
	int i = 0;

	for (; *pkey; ++pkey, ++i) {
		memcpy(pretkey, dkey + (*pkey)->offset, 5);
		css_titlekey(pretkey, (*pkey)->key, 0);

		memcpy(test, dkey, 5);
		css_titlekey(test, pretkey, 0);

		if (memcmp(test, pretkey, 5) == 0) {
			fprintf(stderr, "Using Key %d\n", i+1);
			break;
		}
	}

	if (!*pkey) {
		fprintf(stderr, "Shit - Need Key %d\n", i+1);
		return 0;
	}

	css_titlekey(tkey, pretkey, 0xff);
	return 1;
}

/*
 *
 * this function does the actual descrambling
 *
 * sec: encrypted sector (2048 bytes)
 * key: decrypted title key obtained from css_decrypttitlekey
 *
 */

void css_descramble(byte *sec,byte *key)
{

#define SALTED(i) (key[i] ^ sec[0x54 + (i)])

	unsigned char *end = sec + 0x800;
	int            val;
	unsigned int   lfsr0, lfsr1;
	byte           o_lfsr0, o_lfsr1;

	lfsr0 = ((SALTED(4) << 17) | (SALTED(3) << 9) | (SALTED(2) << 1)) + 8 - (SALTED(2)&7);
       	lfsr0 = (reverse[lfsr0&0xff]<<17) | (reverse[(lfsr0>>8)&0xff] << 9)
		  | (reverse[(lfsr0>>16)&0xff]<<1) |(lfsr0>>24);

	lfsr1 = (reverse[SALTED(0)] << 9) | 0x100 | (reverse[SALTED(1)]);

#if (CSS_DEBUG & 0x10)
		fprintf( stderr,"SEED lfsr0:0x%08x lfsr1: 0x%08x\n", lfsr0, lfsr1);
#endif

	sec+=0x80;
	val = 0;
	while (sec != end) {
   		o_lfsr0 = (lfsr0 >> 12) ^ (lfsr0 >> 4) ^  (lfsr0 >> 3) ^ lfsr0;

		o_lfsr1 = ((lfsr1 >> 14) & 7) ^ lfsr1;
		o_lfsr1 ^= (o_lfsr1 << 3) ^ (o_lfsr1 << 6);

		lfsr1 = (lfsr1 >> 8) ^ (o_lfsr1 << 9);
		lfsr0 = (lfsr0 >> 8) ^ (o_lfsr0 << 17);

   		val += o_lfsr0 + (byte)~o_lfsr1;
		*sec++ = csstab1[*sec] ^ (val & 0xff);
		val >>= 8;

#if (CSS_DEBUG & 0x10)
		fprintf( stderr,"lfsr0:0x%08x lfsr1: 0x%08x o_lfsr0:0x%02x o_lfsr1:0x%02x\n",
				lfsr0, lfsr1, o_lfsr0, o_lfsr1);
#endif
	}
}
